/*
 * $Log$
 * Revision 1.7  2006/07/02 04:51:47  branaway
 * trivial
 *
 * Revision 1.6  2005/09/20 14:56:43  branaway
 * added setWebScopeSession and setWebScopeRequest to the config file, so later on we can use that informaion in jacn unified JSF and Spring
 *
 * Revision 1.5  2005/09/04 10:23:46  branaway
 * support referencing factorybean instance itself
 *
 * Revision 1.4  2005/08/08 17:09:07  branaway
 * added support for adding post processing functional calls.
 * removed web dir since the web is now in another module called htdocs
 *
 * Revision 1.3  2005/08/03 12:42:48  branaway
 * added support for web application context integration: JacnWebApplicationContext
 *
 */
package bran.spring.jacn;

import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.beans.factory.xml.XmlBeanDefinitionReader;
import org.springframework.context.support.AbstractRefreshableApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;

/**
 * Standalone JACN application context, taking the context definition
 * class files from the class path. It can also take xml file resources as well, 
 * allowing mixing Jacn and xml bean definitions.  
 *
 *
 * @author Bing Ran, bran@ccniit.com
 * @see JacnBeanDefinitionReader
 * @see ClassPathXmlApplicationContext
 */
public class JacnApplicationContext extends AbstractRefreshableApplicationContext 
{

	
	private Class[] configClasses;
	private JacnBeanDefinitionReader jacnReader;
	private String[] xmlConfigs;

	// contains <beanName, scope>
	HashMap beanScopeMap = new HashMap();

	/**
	 * Create a new ApplicationContext, loading the definitions
	 * from the given config class file and automatically refreshing the context.
	 * @param configClass config class
	 */
	public JacnApplicationContext(Class configClass) throws BeansException {
		this(configClass, true);
	}

	/**
	 * Create a new ApplicationContext, loading the definitions
	 * from the given config class file and automatically refreshing the context.
	 * @param configs config classes
	 */
	public JacnApplicationContext(Class[] configs) throws BeansException {
		this(configs, true);
	}
	
	/**
	 * Create a new JacnApplicationContext, loading the definitions
	 * from the given Jacn configuration class.
	 * @param configClass configuration class
	 * @param refresh whether to automatically refresh the context,
	 * loading all bean definitions and creating all singletons.
	 * Alternatively, call refresh manually after further configuring the context.
	 * @see #refresh
	 */
	public JacnApplicationContext(Class configClass, boolean refresh) throws BeansException {
		this.configClasses = new Class[] {configClass};
		if (refresh) {
			refresh();
		}
	}

	/**
	 * Create a new JacnApplicationContext, loading the definitions
	 * from the given Jacn configuration classes.
	 * @param configClasses configuration classes
	 * @param refresh whether to automatically refresh the context,
	 * loading all bean definitions and creating all singletons.
	 * Alternatively, call refresh manually after further configuring the context.
	 * @see #refresh
	 */
	public JacnApplicationContext(Class[] configs, boolean refresh) {
		this.configClasses = configs;
		if (refresh) {
			refresh();
		}
	}

	/**
	 * this constructor allows one to configure the applicationContext with mixed
	 * Jacn and xml configuration. 
	 * 
	 * @param jacnConfigs the jacn configuration classes
	 * @param xmlConfigLocations the classical XML configuration locations. 
	 * Pseudo url format accepted. 
	 * @param refresh read the bean defs right away if true; or you must call {@link refresh()}
	 * to load the bean definitions. 
	 */
	public JacnApplicationContext(Class[] jacnConfigs, String[] xmlConfigLocations, boolean refresh) {
		this.configClasses = jacnConfigs;
		this.xmlConfigs = new String[xmlConfigLocations.length];
		for (int i = 0; i < xmlConfigLocations.length; i++) {
			String loc = xmlConfigLocations[i].trim();
			if (!loc.endsWith(".xml")) {
				throw new RuntimeException("xmlConfigLocations parameter takes xml path only");
			}
			xmlConfigs[i] = loc;
		}
		if (refresh) {
			refresh();
		}
	}

	/**
	 * Loads the bean definitions via an JacBeanDefinitionReader.
	 * @see JacnBeanDefinitionReader
	 * @see #loadBeanDefinitions
	 */
	protected void loadBeanDefinitions(DefaultListableBeanFactory beanFactory) throws IOException {
		jacnReader = new JacnBeanDefinitionReader(beanFactory);
		if (this.pcl != null) {
			jacnReader.setPcl(getPcl());
		}
		jacnReader.setResourceLoader(this);

		if(configClasses != null) {
			for (int i = 0; i < this.configClasses.length; i++) {
				Class cls = this.configClasses[i];
				jacnReader.loadBeanDefinitions(getResource(cls));
				Map m = jacnReader.getBeanScopeMap();
				beanScopeMap.putAll(m);
			}
		}
		
		if (this.xmlConfigs != null && this.xmlConfigs.length > 0) {
			XmlBeanDefinitionReader xmlReader = new XmlBeanDefinitionReader(beanFactory);
			xmlReader.setResourceLoader(this);
			for (int i = 0; i < xmlConfigs.length; i++) {
				xmlReader.loadBeanDefinitions(getResources(xmlConfigs[i]));
			}
		}
	}

	private Resource getResource(Class cls) {
		String cc = cls.getName().replace('.', '/') + ".class";
		return new ClassPathResource(cc);
	}

//	public Resource getResource(String arg0) {
//		return super.getResource(arg0);
//	}

//	public String getXml() {
//		reader.getXmlText();
//		return null;
//	}

	PropertyChangeListener pcl;

	public PropertyChangeListener getPcl() {
		return pcl;
	}

	/**
	 * set a PropertyChangeListener that be passed an propertychangevent that holds the 
	 * XML version of the bean definitions in the new value parameter. 
	 * @param listener the listener to receive the bean definitions in XML format.  
	 */
	public void setPropertyChangeListener(PropertyChangeListener listener) {
		this.pcl = listener;
	}

	public String getScope(String beanName) {
		return (String) beanScopeMap.get(beanName);
	}

}
