package bran.spring.jacn;

import java.beans.PropertyChangeEvent;
import java.util.Map;

import javax.faces.FactoryFinder;
import javax.faces.application.Application;
import javax.faces.application.ApplicationFactory;
import javax.faces.context.FacesContext;
import javax.faces.el.ReferenceSyntaxException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.TypeMismatchException;
import org.springframework.beans.factory.BeanCreationException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.beans.factory.support.RootBeanDefinition;

import bran.spring.jacn.faces.Flow;
import bran.spring.jacn.faces.FlowController;
import bran.spring.jacn.faces.FlowManager;

public class SpringFacesBeanFactory extends DefaultListableBeanFactory {
	private static final Log log = LogFactory.getLog(SpringFacesBeanFactory.class);

	JacnWebApplicationContext wctx;

	public SpringFacesBeanFactory(JacnWebApplicationContext jwc, BeanFactory parentFactory) {
		super(parentFactory);
		this.wctx = jwc;
	}

	/**
	 * The reason that I need to control the scope at the bean creation level is because I need to
	 * intercept the creation event on the dependent longer duration scoped beans and retrive them
	 * from the scope rather than creating a new instance. e.g., a request scoped bean depending on
	 * a session scoped bean. At the request of creating a new instance of the request bean, a new
	 * instance of the dependent bean will be created too if I don't work at the object creation
	 * level.
	 * 
	 * Another issue we need to take care is the proxy that we use to apply aspects, or generally
	 * the factorybean way of creating new instances. The factorybean is always singleton while the
	 * product may be of prototype. I'll wait to see if this is the best place to deal with it.
	 * 
	 * @see org.springframework.beans.factory.support.AbstractAutowireCapableBeanFactory#createBean(java.lang.String,
	 *      org.springframework.beans.factory.support.RootBeanDefinition, java.lang.Object[],
	 *      boolean)
	 * 
	 */
	protected final Object createBean(String name, RootBeanDefinition beanDefinition, Object[] args)
			throws BeansException {
		if (log.isDebugEnabled()) {
			log.debug(name);

		}

		FacesContext context = FacesContext.getCurrentInstance();

		if (context == null) {
			// not in JSF
			if (log.isDebugEnabled()) {
				log.debug("no facescontext found. use default createbean()");
			}
			return super.createBean(name, beanDefinition, args);
		} else {
			Map viewAttrs = context.getViewRoot().getAttributes();
			String flowid = (String) viewAttrs.get(FlowManager.FLOW_ID);

			Object bean = null;
			if (!wctx.containsBean(name)) {
				// the searching into the parent will be handled by the applicationContext. 
				
				throw new NoSuchBeanDefinitionException(name, "see Jacn config files");
				
//				BeanFactory parent = getParentBeanFactory();
//				if (parent == null || !parent.containsBean(name)) {
//					throw new NoSuchBeanDefinitionException(name, "see Jacn config files");
//				}
//				else {
//					return parent.
//				}
			} else {

				String scope = wctx.getScope(name);
				if (log.isDebugEnabled()) {
					log.debug("scope of " + name + ": \"" + scope + "\"");
				}

				if (scope != null) {
					if (scope.equals(WebScope.APPLICATION_SCOPE)) {
						bean = context.getExternalContext().getApplicationMap().get(name);
						if (log.isDebugEnabled()) {
							log.debug(bean == null ? "not exists in scope; creaing new..." : "already in scope");
						}
						if (bean == null) {
							bean = super.createBean(name, beanDefinition, args);
							context.getExternalContext().getApplicationMap().put(name, bean);
						}
					} else if (scope.equals(WebScope.SESSION_SCOPE)) {
						bean = context.getExternalContext().getSessionMap().get(name);
						if (log.isDebugEnabled()) {
							log.debug(bean == null ? "not exists in scope; creaing new..." : "already in scope");
						}
						if (bean == null) {
							bean = super.createBean(name, beanDefinition, args);
							context.getExternalContext().getSessionMap().put(name, bean);
						}
					} else if (scope.equals(WebScope.REQUEST_SCOPE)) {
						if (log.isDebugEnabled()) {
							log.debug(bean == null ? "not exists in scope; creaing new..." : "already in scope");
						}
						bean = context.getExternalContext().getRequestMap().get(name);
						if (bean == null) {
							bean = super.createBean(name, beanDefinition, args);
							context.getExternalContext().getRequestMap().put(name, bean);
						}
					} else {
						if (flowid != null) {
							Class clz = beanDefinition.getBeanClass();
							if (clz == FlowManager.class) {
								// somthing is wrong
								// clear any flow id
								viewAttrs.remove(FlowManager.FLOW_ID);
								context.getExternalContext().getRequestMap().remove(FlowManager.FLOW_ID);
								log.warn("flow manager is null while the current session has id: " + flowid);
								log.warn("clear any id");
								bean = super.createBean(name, beanDefinition, args);
							} else {

								if (log.isDebugEnabled()) {
									log.debug("\"none\" scoped; in flow-id ");
								}
								FlowManager fm = FlowManager.getCurrentInstance(context);
								// Flow flow = fm.getFlow(flowid);
								bean = fm.getBean(name);
								// search bean recursive in parent scopes, as in a closure
								// nono, don't do recursive. We might get the same flowcontroller
								// where we really need new instance of it
								// bean = fm.getBeanRecursive(name);

								if (bean == null) {
									if (log.isDebugEnabled()) {
										log.debug("not in flow cache. create new");
									}
									bean = super.createBean(name, beanDefinition, args);
									fm.setBean(name, bean);
								} else {
									if (log.isDebugEnabled()) {
										log.debug("retrieved from flow cache. ");
									}
								}
							}
						} else {
							if (log.isDebugEnabled()) {
								log.debug("\"none\" scoped; not in flow-id. call super.createBean ");
							}
							bean = super.createBean(name, beanDefinition, args);
						}
					}
				} else {
					// this is possible when the bean is defined in classic XML
					if (log.isDebugEnabled()) {
						log.debug("non scope information. default to super.createBean().");
					}
					bean = super.createBean(name, beanDefinition, args);
					// throw new RuntimeException("a jacn bean has no scope
					// information: "
					// + name);
				}
			}

			return bean;
		}
	}

	protected final Object doTypeConversionIfNecessary(Object value, Class targetType) throws TypeMismatchException {
		if (log.isDebugEnabled()) {
			log.debug("doTypeConversionIfNecessary: " + value);
		}
		// we need to resolve value bindings for JSF
		if (value != null && value instanceof String) {
			String sv = (String) value;
			sv = sv.trim();
			if (sv.startsWith("#{") && sv.endsWith("}")) {
				try {
					ApplicationFactory factory = (ApplicationFactory) FactoryFinder
							.getFactory(FactoryFinder.APPLICATION_FACTORY);
					if (factory != null) {
						Application facesApplication = (factory).getApplication();
						if (facesApplication != null) {
							if (log.isDebugEnabled()) {
								log.debug("return value from facescontext value binding");
							}
							return facesApplication.createValueBinding((String) value).getValue(
									FacesContext.getCurrentInstance());
						} else {
							log.debug("could not find faces application");
						}

						// can I use :
						// FacesContext facesContext =
						// FacesContext.getCurrentInstance();
						// return
						// facesContext.getApplication().createValueBinding((String)
						// value).getValue(facesContext);

					} else {
						// not in JSF
						return super.doTypeConversionIfNecessary(value, targetType);
					}
				} catch (ReferenceSyntaxException ex) {
					throw new TypeMismatchException(new PropertyChangeEvent("unknown", null, null, value), targetType,
							ex);
				}
			}

		}
		return super.doTypeConversionIfNecessary(value, targetType);
	}

}